#!/usr/bin/python
import string
import irc
import irc_interface
import re
import md5
import traceback
import registry

caps_re = re.compile("[A-Z]")

from Sync import Synchronized

class BoundIRC:
	def __init__(self):
		self.__interface = irc_interface.Interface(self)
		self.__binds = registry.registry()


	def fileno(self):
		return self.__interface.fileno()


	def connect(self, server, port, nick, gecos):
		self.__interface.connect(server, port, nick, gecos)


	def handle_input(self):
		self.__interface.handle_input()


	def bind(self, message, function, regexp=""):
		"""Bind a function to a server message
		
		You pass in the server message name, the function you want to
		call, and optionally a regular expression to match the text
		portion of the message.  If the regular expression doesn't
		match, the function isn't called.  If no regular expression is
		provided, the function is always called.
		
		Functions will be called with the args as the first argument,
		and the text as the second.

		"""

		if caps_re.search(regexp):
			raise ValueError, "capital letters are not allowed in a regexp"
		message = irc.lower(message)
		tup = (re.compile(regexp), function,
			   md5.new(regexp).digest())
		self.__binds.add(message, tup)


	def unbind(self, message, function, regexp=""):
		"""Unbind a function from a server message

		If you want to  remove every binding of a function from a server
		message, specify None for the regexp.
		
		"""

		if regexp == None:
			digest = None
		else:
			digest = md5.new(regexp).digest

		message = irc.lower(message)
		bindings = self.__binds[message]
		for i in bindings:
			if i[0] == function:
				if digest and digest != i[2]:
					continue
			self.__binds.remove(message, i)

	def handle_message(self, op, sender, recip, forum, addl, args, text):
		if addl:
			s = irc.lower(string.rstrip(addl[0])) #We strip RH space all msgs come with \r\n on them
		else:
			s = ""

		lop = irc.lower(op)
		if self.__binds.has_key(lop):
			for i in (self.__binds[lop]):
				if i[0].search(s):
					try:
						i[1](sender, recip, forum, addl, (op, args, text))
					except:
						traceback.print_exc()
	
	def write(self, args, *text):
		apply(self.__interface.write, (args,) + tuple(text))


	def ctcp_pong(sender, recip, forum, addl, raw):
		"""Respond to a CTCP PING

		Clients can also ping you using the Client-To-Client Protocol.
		This replies to them.  Bind this with:
		
		bot.bind("CPRIVMSG", ctcp_pong, "^ping ")

		"""
		sender.notice("\001PING %s\001" % addl[1])
